﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
//
using System.IO.Ports;	//SerialPort
using System.Threading;	//Sleep()
using System.Globalization;	//Calendar()

namespace LcdClockControler
{
	public partial class FormMain : Form
	{
		private const string
			APP_DATE	= "2009/08",
			APP_VER		= "ver 1.00b",
			APP_AUTHOR	= "『昼夜逆転』工作室 J.Sugita";
		private static readonly string
			APP_ABOUT	= APP_DATE + "  " + APP_VER + Environment.NewLine + APP_AUTHOR;
		
		private CdcIO cdcio;
		private LcdClock lcdClock;
		private bool isConnect;
		
		//コンストラクタ
		public FormMain()
		{
			InitializeComponent();
			lblVersion.Text = APP_VER;
			
			cdcio = new CdcIO();
			lcdClock = new LcdClock();
			isConnect = false;

			//日時のリストボックスのItemsを登録し、描画更新を開始する
			setDateControlValue();
			drawNowDateTime();
			timer.Enabled = true;
			
			//COMポートを探す
			if (comPortListup())
			{
				setControlEnabled();
				cbCOMPortList.Enabled = true;
				btnConnect.Enabled = true;
				info("COMポートを指定し, 接続してください.");
			}
			else
			{
				info("COMポートが見つかりません.");
			}
		}

		//about
		private void lblVersion_Click(object sender, EventArgs e)
		{
			lblVersion.Text = APP_ABOUT;
		}
		
		//テスト
		private void lblInfo_Click(object sender, EventArgs e)
		{
			if (!isConnect) return;
			
			//AVR-CDCのポートを設定する
			cdcio.Set(SFR.DDRB, 1 << RB.PB0 | 1 << RB.PB1);

			//LEDチカチカ
			for (int i = 0; i < 5; i++)
			{
				cdcio.Set(SFR.PORTB, 1 << RB.PB0);
				Thread.Sleep(200);
				cdcio.Set(SFR.PORTB, 1 << RB.PB1);
				Thread.Sleep(200);
			}

			//相手側の受信バッファをクリアする
			cdcio.sendByte(CMD.BUF_CLEAR);

			//AVR-CDCのポートを初期値に戻す
			cdcio.Set(SFR.PORTB, 0x00);
			cdcio.Set(SFR.DDRB, 0x00);
		}

		//現在の状態やエラーメッセージを表示する
		public void info(string mes)
		{
			lblInfo.Text = mes;
		}
		
		//COMポートを選択リストにセットする
		private	bool comPortListup()
		{
			cbCOMPortList.Items.Clear();
			foreach (string name in SerialPort.GetPortNames())
				cbCOMPortList.Items.Add(name);

			if (cbCOMPortList.Items.Count == 0)
				return false;
			
			cbCOMPortList.SelectedIndex = cbCOMPortList.Items.Count - 1;
			return true;
		}

		//日時のコントロールのItemsを登録する
		private const string WEEK_NAME = "日月火水木金土";
		private void setDateControlValue()
		{
			//年月日、曜日
			cbYear.Items.Clear();
			for (int i = 0; i <= 99; i++)
				cbYear.Items.Add(string.Format("20{0:d02}", i));
			
			cbMonth.Items.Clear();
			for (int i = 1; i <= 12; i++)
				cbMonth.Items.Add(string.Format("{0:d02}", i));
			
			cbDay.Items.Clear();
			for (int i = 1; i <= 31; i++)
				cbDay.Items.Add(string.Format("{0:d02}", i));
			
			cbWeek.Items.Clear();
			for (int i = 0; i <= 6; i++)
				cbWeek.Items.Add(string.Format("{0}", WEEK_NAME[i]));
			
			//時分秒
			cbHour.Items.Clear();
			for (int i = 0; i <= 23; i++)
				cbHour.Items.Add(string.Format("{0:d02}", i));
			
			cbMin.Items.Clear();
			for (int i = 0; i <= 59; i++)
				cbMin.Items.Add(string.Format("{0:d02}", i));
			
			cbSec.Items.Clear();
			for (int i = 0; i <= 59; i++)
				cbSec.Items.Add(string.Format("{0:d02}", i));
		}

		//コントロールのenabled/disabledを設定する
		private void setControlEnabled()
		{
			cbCOMPortList.Enabled = !isConnect;
			btnConnect.Text = isConnect ? "解除" : "接続";
			pnlClockSetting.Enabled = isConnect;
		}

		//接続開始／接続解除
		private void btnConnect_Click(object sender, EventArgs e)
		{
			lblVersion.Text = APP_VER;
			connectAction();
		}
		private void connectAction()
		{
			if (isConnect)
			{
				if (!doDisconnect()) return;
				
				//コントロールの有効状態を設定する
				setControlEnabled();
			}
			else
			{
				if (!doConnect())
				{
					setControlEnabled();
					return;
				}
				
				//シリアル通信初期化（AVR-CDCのRXD/TXDを使う準備）
				if (!cdcio.initSerial())
				{
					//AVR-CDCではないCOMポートへ接続している可能性が高い
					doDisconnect();
					info("ポートが違うか, AVR-CDCが動作していません.");
					return;
				}
				
				//LCD Clockにcdcioをセットする
				//これ以降、LCD時計を操作してよい
				lcdClock.setCdcIO(cdcio);
				
				//コントロールの有効状態を設定する
				setControlEnabled();
				
				//LCDの現在の状態を取得する
				if (!getLcdStatus())
					pnlClockSetting.Enabled = false; 
			}
		}
		
		//接続開始する
		private bool doConnect()
		{
			string	portName = (string)cbCOMPortList.SelectedItem;
			if (!cdcio.Open(portName))
			{
				info(portName + "のオープン失敗.");
				return false;
			}

			isConnect = true;
			info(portName + "に接続しました.");
			return true;
		}
		
		//LCDの現在の状態を取得する
		private bool getLcdStatus()
		{
			if (!lcdClock.getValue())
			{
				info("現在値が取得できませんでした.");
				return false;
			}
			
			//値をコントロールに反映させる
			valueToControlDisplayMode();
			valueToControlBacklight();
			valueToControlContrast();
			return true;
		}

		//表示モードの値をコントロールに反映させる
		private void valueToControlDisplayMode()
		{
			cbDisplayMode.SelectedIndex = lcdClock.dispMode;
		}
		
		//バックライトの値をコントロールに反映させる
		private void valueToControlBacklight()
		{
			barBackLight.Value = (lcdClock.backlight == 0xFF ? 0x100 : lcdClock.backlight) / 0x10;
			drawBacklightValue();
		}
		
		//コントラストの値をコントロールに反映させる
		private void valueToControlContrast()
		{
			barContrast.Value = lcdClock.contrast / 0x10;
			drawContrastValue();
		}
		
		//接続解除する
		private bool doDisconnect()
		{
			bool	error = false;
			string	portName = (string)cbCOMPortList.SelectedItem;
			
			try
			{
				cdcio.Close();
			}
			catch
			{
				error = true;
				info(portName + "のクローズ失敗.");
			}
			finally
			{
				try
				{
					cdcio.Dispose();
				}
				catch
				{
					if (!error)
					{
						error = true;
						info(portName + "の破棄失敗.");
					}
				}
			}

			isConnect = cdcio.IsOpen;
			if (isConnect)
				error = true;
			else
				info(portName + "を接続解除しました.");
			
			return !error;
		}
		
		//バックライトの明るさ変更
		private void barBackLight_Scroll(object sender, EventArgs e)
		{
			changeBacklight();
		}
		private void changeBacklight()
		{
			int bklt = barBackLight.Value * 0x10;	//{0-16} * 16
			if (bklt == 0x100) bklt = 0xFF;
			if (!lcdClock.setConBkLt(lcdClock.contrast, bklt))
				valueToControlBacklight();
			drawBacklightValue();
		}
		
		//バックライトの値を描画する
		private void drawBacklightValue()
		{
			lblBackLight.Text = "バックライト : " + barBackLight.Value;
		}
		
		//コントラスト変更
		private void barContrast_Scroll(object sender, EventArgs e)
		{
			changeContrast();
		}
		private void changeContrast()
		{
			int con = barContrast.Value * 0x10;	//{0-8} * 16
			if (!lcdClock.setConBkLt(con, lcdClock.backlight))
				valueToControlContrast();
			drawContrastValue();
		}

		//コントラストの値を描画する
		private void drawContrastValue()
		{
			lblContrast.Text = "コントラスト : " + barContrast.Value;
		}

		//表示モード変更
		private void cbDisplayMode_SelectionChangeCommitted(object sender, EventArgs e)
		{
			changeDisplayMode();
		}
		private void changeDisplayMode()
		{
			if (lcdClock.setDisplayMode(cbDisplayMode.SelectedIndex))
				lcdClock.dispMode = cbDisplayMode.SelectedIndex;
			else
				valueToControlDisplayMode();
		}

		//日時設定タイプで「現在の日時」を選択した
		private void rbtnToday_CheckedChanged(object sender, EventArgs e)
		{
			dateReadyToday();
		}
		private void dateReadyToday()
		{
			pnlDate.Enabled = false;
			timer.Enabled = true;
		}
		
		//1秒経過
		private void timer_Tick(object sender, EventArgs e)
		{
			drawNowDateTime();
		}
		
		//現在の時刻をリストボックスに表示する
		private void drawNowDateTime()
		{
			DateTime dt = DateTime.Now;
			cbYear.SelectedIndex = dt.Year % 100;
			cbMonth.SelectedIndex = dt.Month - 1;
			cbDay.SelectedIndex = dt.Day - 1;
			cbWeek.SelectedIndex = (int)dt.DayOfWeek;
			cbHour.SelectedIndex = dt.Hour;
			cbMin.SelectedIndex = dt.Minute;
			cbSec.SelectedIndex = dt.Second;
		}
		
		//日時設定タイプで「任意の日時」を選択した
		private void rbtnUser_CheckedChanged(object sender, EventArgs e)
		{
			dateReadyUser();
		}
		private void dateReadyUser()
		{
			pnlDate.Enabled = true;
			timer.Enabled = false;
		}

		//日時を確定して送信する
		private void btnSetDate_Click(object sender, EventArgs e)
		{
			setLcdClockDate();
		}
		private void setLcdClockDate()
		{
			DateTime dt = DateTime.Now;		//← if (rbtnToday.Checked)
			if (rbtnUser.Checked) dt = cbMakeDataTime();
			
			if (!lcdClock.setDate(dt))
				info("日時の設定に失敗しました.");
		}

		//リストボックスの日時からDateTime構造体を作る
		private DateTime cbMakeDataTime()
		{
			//不正な日付でDateTimeをnewすると例外が発生する
			try
			{
				return new DateTime(
				2000 + cbYear.SelectedIndex,
				cbMonth.SelectedIndex + 1,
				cbDay.SelectedIndex + 1,
				cbHour.SelectedIndex,
				cbMin.SelectedIndex,
				cbSec.SelectedIndex
				);
			}
			catch { return DateTime.Now; }
		}

		//リストボックスで年月日が変更された
		private void cbYear_SelectionChangeCommitted(object sender, EventArgs e)
		{
			changeDate();
		}
		private void cbMonth_SelectionChangeCommitted(object sender, EventArgs e)
		{
			changeDate();
		}
		private void cbDay_SelectionChangeCommitted(object sender, EventArgs e)
		{
			changeDate();
		}
		
		//年月日の妥当性を確認／補正する
		private void changeDate()
		{
			//「2月31日」などの不正な日付を補正する
			Calendar ca = new JapaneseCalendar();
			int lastDay = ca.GetDaysInMonth(2000 + cbYear.SelectedIndex, cbMonth.SelectedIndex + 1);
			if (lastDay < cbDay.SelectedIndex + 1)
				cbDay.SelectedIndex = lastDay - 1;
			
			//日付に合わせて曜日を更新する
			DateTime dt = cbMakeDataTime();
			cbWeek.SelectedIndex = (int)dt.DayOfWeek;
		}

	}
}
